@@ -4,7 +4,6 @@
 
 """
 
-import btk
 import vtk
 import re
 import numpy as np
@@ -14,7 +13,12 @@ import scipy.io as sio
 from openpyxl import load_workbook
 
 
-def readC3D(fileName, sections, opts={}):
+
+def readMocapFile(reader, fileName, sections, 
+              removeSegmentNameFromMarkerNames=True,
+              setMarkersZeroValuesToNaN=True,
+              setMarkersDataFromMeterstoMillimeters=True
+              ):
     """Read C3D file.
 
     Parameters
@@ -50,48 +54,72 @@ def readC3D(fileName, sections, opts={}):
     """
 
     # Open C3D pointer
-    reader = btk.btkAcquisitionFileReader()
-    reader.SetFilename(str(fileName))
-    reader.Update()
-    acq = reader.GetOutput()
+    reader.readFromFile(fileName)
 
     # Initialize data structure
     data = {}
-
-    # Read markers
-    if 'markers' in sections:
-
-        # Convert points unit to mm
-        pointUnit = acq.GetPointUnit()
-        if pointUnit == 'mm':
-            scaleFactor = 1.
-        elif pointUnit == 'm':
+    
+    # Correct for unit if necessary
+    scaleFactor = 1.
+    markerUnit = reader.getVectorUnit('marker')
+    if markerUnit == 'm':
+        if setMarkersDataFromMeterstoMillimeters:
             scaleFactor = 1000.
-        else:
-            raise Exception('Point unit not recognized')
+            markerUnit = 'mm'
+    angleUnit = reader.getVectorUnit('angle')
 
-        # Get relevant marker data (N x 3)
-        markers = {}
-        coll = acq.GetPoints()
-        for i in range(coll.GetItemNumber()):
-            point = coll.GetItem(i)
-            label = point.GetLabel()
-            if 'removeSegmentNameFromMarkerNames' in opts and opts['removeSegmentNameFromMarkerNames']:
+    # Get relevant vector data (N x 3)
+    markers = {}
+    angles = {}
+    for i in range(reader.getNVectors()):
+        label, vecData, vecType = reader.getVector(i)
+        if label is None:
+            continue
+        if vecType == 'marker':
+            if removeSegmentNameFromMarkerNames:
                 labelList = label.split(':')
                 if len(labelList) > 1:
                     label = labelList[1]
-            marker = point.GetValues() * scaleFactor
-            if 'setMarkersZeroValuesToNaN' in opts and opts['setMarkersZeroValuesToNaN']:
-                marker[marker==0.] = np.nan # replace 0. with np.nan
-            markers[label] = marker
+            data_ = vecData * scaleFactor
+            if setMarkersZeroValuesToNaN:
+                data_[data_==0.] = np.nan # replace 0. with np.nan
+            markers[label] = data_
+        elif vecType == 'angle':
+            data_ = vecData
+            angles[label] = data_
+            
+    # Get events
+    events = {}
+    allList = []
+    for i in range(reader.getNEvents()):
+        label, context, frame = reader.getEvent(i)
+        allList.append(frame)
+    events['allMixedList'] = sorted(allList)
+    print(events)
+            
+    freq = reader.getVectorFrequency()
 
-        data['markers'] = markers
+    if 'markers' in sections:
+        data['markers'] = {}
+        data['markers']['data'] = markers
+        data['markers']['unit'] = markerUnit
+        data['markers']['freq'] = freq
+        
+    if 'angles' in sections:
+        data['angles'] = {}
+        data['angles']['data'] = angles
+        data['angles']['unit'] = angleUnit
+        data['angles']['freq'] = freq
+        
+    if 'events' in sections:
+        data['events'] = {}
+        data['events']['data'] = events
 
     # Return data
     return data
-
-
-def writeC3D(fileName, data, copyFromFile=None):
+    
+    
+def writeMocapFile(writer, fileName, data, copyFromFile=None, reader=None):
     """Write to C3D file.
 
     Parameters
@@ -102,16 +130,15 @@ def writeC3D(fileName, data, copyFromFile=None):
     data : dict
         Data dictionary that can contain the following keys:
 
-        - markers: this is marker-related data. This dictionary contains:
-            - data: dictionary where each key is a point label, and each
-              value is a N x 3 np.ndarray of 3D coordinates (in *mm*), where N is
+        - markers or angles: this is vector-related data. This dictionary contains:
+            - data: dictionary where each key is a vector label, and each
+              value is a N x 3 np.ndarray of 3D coordinates, where N is
               the number of time frames. This field is always necessary.
-            - framesNumber: number of data points per marker.
+            - framesNumber: number of data points per vector.
               This field is necessary when creating files from scratch.
-            - unit: string indicating the markers measurement unit. Available
-              strings are 'mm' and 'm'.
+            - unit: string indicating the vectors measurement unit.
               This field is necessary when creating files from scratch.
-            - freq: number indicating the markers acquisition frequency.
+            - freq: number indicating the vectors acquisition frequency.
               This field is necessary when creating files from scratch.
 
     copyFromFile : str
@@ -122,41 +149,59 @@ def writeC3D(fileName, data, copyFromFile=None):
 
     if copyFromFile is not None:
         # Open C3D pointer
-        reader = btk.btkAcquisitionFileReader()
-        reader.SetFilename(str(copyFromFile))
-        reader.Update()
-        acq = reader.GetOutput()
-        if 'markers' in data:
-            nMarkerFrames = acq.GetPointFrameNumber()
-            pointUnit = acq.GetPointUnit()
+        reader.readFromFile(copyFromFile)
+        writer.setData(reader.getData())
+        nVectorFrames = reader.getNVectorFrames()
+        writer.setNVectorFrames(nVectorFrames)
+        markerUnit = reader.getVectorUnit('marker')
+        angleUnit = reader.getVectorUnit('angle')
     else:
         # Create new acquisition
-        acq = btk.btkAcquisition()
-        if 'markers' in data:
-            nMarkerFrames = data['markers']['framesNumber']
-            acq.Init(0, nMarkerFrames)
-            pointUnit = data['markers']['unit']
-            acq.SetPointUnit(pointUnit)
-            pointFreq = data['markers']['freq']
-            acq.SetPointFrequency(pointFreq)
+        writer.initEmpty()
+        for section in data:
+            vecType = section[:-1]
+            if section in ['markers', 'angles']:
+                if 'framesNumber' in data[section]:
+                    nVectorFrames = data[section]['framesNumber']
+                else:
+                    firstVectorName = data[section]['data'].keys()[0]
+                    firstVector = data[section]['data'][firstVectorName]
+                    nVectorFrames = firstVector.shape[0]
+                writer.setNVectorFrames(nVectorFrames)
+                writer.initSpaceForNVectorFrames()
+                vectorUnit = data[section]['unit']
+                if section == 'markers':
+                    markerUnit = vectorUnit
+                elif section == 'angles':
+                    angleUnit = vectorUnit
+                writer.setVectorUnit(vecType, vectorUnit)
+                vectorFreq = data[section]['freq']
+                writer.setVectorFrequency(vectorFreq)
 
-    if 'markers' in data:
-        # Write marker data
-        markers = data['markers']['data']
-        for m in markers:
-            newMarker = btk.btkPoint(m, nMarkerFrames)
-            if pointUnit == 'm':
-                markerData = markers[m] / 1000.
-            elif pointUnit == 'mm':
-                markerData = markers[m].copy()
-            newMarker.SetValues(markerData)
-            acq.AppendPoint(newMarker)
+    for section in data:
+        if section in ['markers', 'angles']:
+            vecType = section[:-1]
+            # Write vector data
+            vectors = data[section]['data']
+            for v in vectors:
+                newVectorUnit = data[section]['unit']
+                if section == 'markers':
+                    if markerUnit == newVectorUnit:
+                        vectorData = vectors[v].copy()
+                    else:
+                        if vectorUnit == 'm' and newVectorUnit == 'mm':
+                            vectorData = vectors[v] / 1000.
+                        else:
+                            raise Exception('not solvable conflict between existing and new markers unit')
+                elif section == 'angles':
+                    if angleUnit == newVectorUnit:
+                        vectorData = vectors[v].copy()
+                    else:
+                        raise Exception('not solvable conflict between existing and new angles unit')
+                writer.addVector(v, vectorData, vecType)
 
     # Write to C3D
-    writer = btk.btkAcquisitionFileWriter()
-    writer.SetInput(acq)
-    writer.SetFilename(str(fileName))
-    writer.Update()
+    writer.writeToFile(fileName)
     
 
 def readMimics(fileName, sections):
