@@ -10,19 +10,21 @@ from . import fio, kine, spine
 
 import os
 from mpl_toolkits.mplot3d import Axes3D
+import matplotlib
+from .pyver import __pyver
+if __pyver == 3:
+    matplotlib.use('Qt5Agg')
+else:
+    matplotlib.use('Qt4Agg')
 import matplotlib.pyplot as plt
-
+from . import utils
    
     
     
     
 
-def performSpineAnalysis(
-                        markers1,
-                        markers2,
-                        singleMarkersDescFile,
-                        segmentsDescFile,
-                        clustersDescDir,
+def performSpineAnalysis1(
+                        markers,
                         spinePointNames,
                         resultsDir,
                         anglesDef = {},
@@ -31,175 +33,17 @@ def performSpineAnalysis(
                         froSpineSplineOrder=3,
                         savePlots=True,
                         ):
-                            
-    # calculate THORAX anatomical reference frame for acquisition system 1
-    if set(['CLAV','STRN','C7','T9']) <= set(markers1.keys()):
-        print('Creating thorax reference frame for acquisition system 1 with 4 markers')
-        markers1Tho = {}
-        markers1Tho['IJ'] = markers1['CLAV']
-        markers1Tho['PX'] = markers1['STRN']
-        markers1Tho['C7'] = markers1['C7']
-        markers1Tho['T8'] = markers1['T9']
-        RTho1, OTho1 = kine.thoraxPoseISB(markers1Tho)
-        RTTho1 = kine.composeRotoTranslMatrix(RTho1, OTho1)
-        RTTho1i = kine.inv2(RTTho1)
-        isTho1Visible = True
-        tho1Markers = ['CLAV','STRN','C7','T9']
+
+    # Calculate PELVIS anatomical reference frame for acquisition system
+    allPelvisVisible = set(['RASI','LASI','RPSI','LPSI']) <= set(markers.keys())
+    if allPelvisVisible:
+        RPel, OPel = kine.pelvisPoseISB(markers, s='R')
     else:
-        markers1Tho = {}
-        for m in ['CLAV','STRN','C7','T9']:
-            if m in markers1:
-                markers1Tho[m] = markers1[m]
-        if len(markers1Tho.keys()) == 3:
-            print('Creating thorax reference frame for acquisition system 1 with 3 markers')
-            RTho1, OTho1 = kine.markersClusterFun(markers1Tho, markers1Tho.keys())
-            RTTho1 = kine.composeRotoTranslMatrix(RTho1, OTho1)
-            RTTho1i = kine.inv2(RTTho1)
-            isTho1Visible = True
-            tho1Markers = markers1Tho.keys()
-        else:
-            print('Cannot create thorax reference frame for acquisition system 1')
-            RTTho1i = None
-            isTho1Visible = False
+        RPel, OPel = kine.pelvisPoseNoOneASI(markers, s='R')
+    RTPel = kine.composeRotoTranslMatrix(RPel, OPel)
+    RTPeli = kine.inv2(RTPel)
     
-    # calculate PELVIS anatomical reference frame for acquisition system 1
-    if set(['RASI','LASI','RPSI','LPSI']) <= set(markers1.keys()):
-        print('Creating pelvis reference frame for acquisition system 1 with 4 markers')
-        RPel1, OPel1 = kine.pelvisPoseISB(markers1, s='R')
-        RTPel1 = kine.composeRotoTranslMatrix(RPel1, OPel1)
-        RTPel1i = kine.inv2(RTPel1)
-        isPel1Visible = True
-        pel1Markers = ['RASI','LASI','RPSI','LPSI']
-    else:
-        markers1Pel = {}
-        for m in ['RASI','LASI','RPSI','LPSI']:
-            if m in markers1:
-                markers1Pel[m] = markers1[m]
-        if len(markers1Pel.keys()) == 3:
-            print('Creating pelvis reference frame for acquisition system 1 with 3 markers')
-            RPel1, OPel1 = kine.pelvisPoseNoOneASI(markers1Pel, s='R')
-            RTPel1 = kine.composeRotoTranslMatrix(RPel1, OPel1)
-            RTPel1i = kine.inv2(RTPel1)
-            isPel1Visible = True
-            pel1Markers = markers1Pel.keys()
-        else:
-            print('Cannot create pelvis reference frame for acquisition system 1')
-            RTPel1i = None
-            isPel1Visible = False
-    
-    # calculate THORAX anatomical reference frame for acquisition system 2
-    if len(tho1Markers) == 4:
-        markers2Tho = {}
-        markers2Tho['IJ'] = markers2['CLAV']
-        markers2Tho['PX'] = markers2['STRN']
-        markers2Tho['C7'] = markers2['C7']
-        markers2Tho['T8'] = markers2['T9']
-        RTho2, OTho2 = kine.thoraxPoseISB(markers2Tho)
-    else:
-        RTho2, OTho2 = kine.markersClusterFun(markers2, tho1Markers)
-    RTTho2 = kine.composeRotoTranslMatrix(RTho2, OTho2)
-    
-    # calculate PELVIS anatomical reference frame for acquisition system 2
-    if len(pel1Markers) == 4:
-        RPel2, OPel2 = kine.pelvisPoseISB(markers2, s='R')
-    else:
-        RPel2, OPel2 = kine.pelvisPoseNoOneASI(markers2, s='R')
-    RTPel2 = kine.composeRotoTranslMatrix(RPel2, OPel2)
-    RTPel2i = kine.inv2(RTPel2)
-    
-    # Handles single markers
-    markers2New = markers2.copy()
-    singleMarkersInfo = fio.readStringListMapFile(singleMarkersDescFile)
-    for m in singleMarkersInfo:
-        markerName = m
-        segment = singleMarkersInfo[m][0]
-        if len(singleMarkersInfo[m]) > 1:
-            rawMarkerName = singleMarkersInfo[m][1]
-        else:
-            rawMarkerName = markerName
-        
-        # Express real marker in anatomical reference frame 
-        if segment == 'thorax':
-            RT1i = RTTho1i
-        elif segment == 'pelvis':
-            RT1i = RTPel1i
-        else:
-            raise Exception('segment must be one of: thorax, pelvis')
-        
-        if RT1i is None:
-            print('single marker %s cannot be corrected: its local position in %s cannot be calculated, %s missing' % (markerName, segment, segment))
-            continue
-        
-        targetMarkers = {}
-        targetMarkers[markerName + ' base'] = markers1[rawMarkerName + ' base']
-        targetMarkers['True ' + markerName] = markers1['True ' + rawMarkerName]
-        targetMarkersLoc = kine.changeMarkersReferenceFrame(targetMarkers, RT1i)
-        
-        if segment == 'thorax':
-            RT2 = RTTho2
-        elif segment == 'pelvis':
-            RT2 = RTPel2
-        markers2New.update(kine.changeMarkersReferenceFrame(targetMarkersLoc, RT2))
-    
-    if isTho1Visible and isPel1Visible:
-    
-        # Handles clusters
-        segmentsInfo = fio.readStringListMapFile(segmentsDescFile)
-        
-        for s in segmentsInfo:
-            segmentName = s
-            clusterType = segmentsInfo[s][0]
-            print('%s %s' % (segmentName, clusterType))
-            
-            # Read data for cluster connected to current segment
-            clusterInfo = fio.readStringListMapFile(os.path.join(clustersDescDir, clusterType + '.txt'))
-            clusterInfo = {m: np.array(clusterInfo[m]) for m in clusterInfo}
-            
-            # Modify cluster marker names with real ones
-            clusterInfoSpec = clusterInfo.copy()
-            subs = segmentsInfo[s][1:]
-            clusterBaseName = clusterBaseNameSpec = 'Base'
-            for sub in subs:
-                s = sub.split(':')
-                if s[0] == clusterBaseName:
-                    clusterBaseNameSpec = s[1]
-                del clusterInfoSpec[s[0]]
-                clusterInfoSpec[s[1]] = clusterInfo[s[0]]
-            
-            # SVD for acquisition system 1
-            args = {}
-            markersLoc = clusterInfoSpec.copy()
-            args['mkrsLoc'] = markersLoc
-            args['verbose'] = True
-            args['useOriginFromTrilat'] = False
-            mkrList = markersLoc.keys()
-            R1, T1, infoSVD1 = kine.rigidBodySVDFun2(markers1, mkrList, args)
-            RT1 = kine.composeRotoTranslMatrix(R1, T1)
-            
-            # Express cluster base and real marker in cluster reference frame
-            RT1i = kine.inv2(RT1)
-            targetMarkers = {}
-            targetMarkers[clusterBaseNameSpec + ' 1'] = markers1[clusterBaseNameSpec]
-            targetMarkers['True ' + segmentName] = markers1['True ' + segmentName]
-            targetMarkersLoc = kine.changeMarkersReferenceFrame(targetMarkers, RT1i)
-            
-            # SVD for acquisition system 2
-            clusterBaseMarkerLoc = clusterInfoSpec[clusterBaseNameSpec]
-            args = {}
-            markersLoc = clusterInfoSpec.copy()
-            del markersLoc[clusterBaseNameSpec]
-            args['mkrsLoc'] = markersLoc
-            args['verbose'] = True
-            args['useOriginFromTrilat'] = False
-            mkrList = markersLoc.keys()
-            R2, T2, infoSVD2 = kine.rigidBodySVDFun2(markers2, mkrList, args)
-            RT2 = kine.composeRotoTranslMatrix(R2, T2)
-            
-            # Copy and rename cluster base marker to be expressed in system 2 in global reference frame
-            targetMarkersLoc[clusterBaseNameSpec + ' 2'] = clusterBaseMarkerLoc
-            
-            # Update list of markers for system 2 in global reference frame
-            markers2New.update(kine.changeMarkersReferenceFrame(targetMarkersLoc, RT2))
+    markersNew = markers.copy()
     
     # Express spine points in pelvis reference frame
     spinePointNamesNew = spinePointNames[:]
@@ -207,18 +51,18 @@ def performSpineAnalysis(
     spinePointNamesNewAll = spinePointNamesNew + AIAPointNames
     for i, m in enumerate(spinePointNames):
         # If wanted point name does not exist
-        if m not in markers2New:
+        if m not in markersNew:
             # If wanted point is a True-type point
             if m.find('True') > -1:
                 spinePointNamesNew[i] = m[5:]
-    spinePoints = {m: markers2New[m] for m in spinePointNamesNew}
-    spinePointsPel = kine.changeMarkersReferenceFrame(spinePoints, RTPel2i)
+    spinePoints = {m: markersNew[m] for m in spinePointNamesNew}
+    spinePointsPel = kine.changeMarkersReferenceFrame(spinePoints, RTPeli)
     
     # Create extra points of interest
     extraPoints = {}
-    extraPoints['C7 for CVA'] = markers2New['True C7'].copy()
-    extraPoints['C7 for CVA'][:,2] = markers2New['True SACR'][:,2]
-    extraPointsPel = kine.changeMarkersReferenceFrame(extraPoints, RTPel2i)
+    extraPoints['C7 for CVA'] = markersNew['True C7'].copy()
+    extraPoints['C7 for CVA'][:,2] = markersNew['True SACR'][:,2]
+    extraPointsPel = kine.changeMarkersReferenceFrame(extraPoints, RTPeli)
     
     # Merge spine points in one array
     spineData = np.stack([spinePointsPel[m] for m in spinePointNamesNew], axis=2)  # Nf x 3 x Np
@@ -226,7 +70,7 @@ def performSpineAnalysis(
     
     # Init results
     res = {}
-    res['newMarkers'] = markers2New
+    res['newMarkers'] = markersNew
     res['newMarkers'].update(extraPoints)
     sup, inf = spinePointNamesNew[:-1], spinePointNamesNew[1:]
     spineAngleNames = [sup[i] + '_' + inf[i] for i in range(len(sup))]
@@ -290,7 +134,7 @@ def performSpineAnalysis(
             ndxDer2ChangeSign = np.append(np.diff(np.sign(der2Dense)), [False]) != 0
             win = np.cumsum(ndxDer1ChangeSign)
             win[win != 1] = 0
-            win = win.astype(np.bool)
+            win = win.astype(bool)
             ndxDer2ChangeSign = ndxDer2ChangeSign & win
             if ndxDer2ChangeSign.sum() != 1:
 #                raise Exception('sagittal: there seems to be not exactly 1 inflection point')
@@ -395,7 +239,7 @@ def performSpineAnalysis(
 #            plt.show()
             
             # Plot spine in sagittal/frontal plane
-            fig = plt.figure()
+            plt.clf()
             
             plt.subplot(1, 2, 1)
             plt.plot(spineDataSag[:,1], spineDataSag[:,0], 'o')
@@ -436,17 +280,13 @@ def performSpineAnalysis(
             
             plt.savefig(os.path.join(figuresDir, 'tf_%04d.png' % i), format='png', orientation='landscape')
     
+    plt.close('all')
+    
     # Create MATLAB-friendly reslts structure
-    def adjustDictForML(d):
-        dML = {}
-        for k in d:
-            kML = k.replace(' ', '_')
-            dML[kML] = d[k]
-        return dML
     resML = res.copy()
-    resML['newMarkers'] = adjustDictForML(res['newMarkers'])        
-    resML['spineAngles']['sagittal'] = adjustDictForML(res['spineAngles']['sagittal'])
-    resML['spineAngles']['frontal'] = adjustDictForML(res['spineAngles']['frontal'])
+    resML['newMarkers'] = utils.adjustDictForML(res['newMarkers'])        
+    resML['spineAngles']['sagittal'] = utils.adjustDictForML(res['spineAngles']['sagittal'])
+    resML['spineAngles']['frontal'] = utils.adjustDictForML(res['spineAngles']['frontal'])
     
     # Save data to MAT file
     fio.writeMATFile(os.path.join(resultsDir, 'results.mat'), resML)
@@ -456,5 +296,137 @@ def performSpineAnalysis(
     
     
 
+
+def performSpineAnalysis2(
+                        markers,
+                        resultsDir,
+                        frames = None,
+                        savePlots=True,
+                        ):
     
-    
\ No newline at end of file
+    markers2 = markers.copy()
+    
+    # Calculate PELVIS anatomical reference frame for acquisition system
+    allPelvisVisible = set(['RASI','LASI','RPSI','LPSI']) <= set(markers.keys())
+    if allPelvisVisible:
+        RPel, OPel = kine.pelvisPoseISB(markers, s='R')
+    else:
+        raise Exception('any of RASI, LASI RPSI LPSI not existing')
+    RTPel = kine.composeRotoTranslMatrix(RPel, OPel)
+    RTPeli = kine.inv2(RTPel)
+    
+    # Calculate virtual sacrum point
+    markersNew = {}
+    markersNew['SACR'] = 0.5 * (markers['RPSI'] + markers['LPSI'])
+    markersNew['L4'] = 0.5 * (markers['RL4'] + markers['LL4'])
+    markers2.update(markersNew)
+    
+    # Express points in PELVIS anatomical reference frame
+    pointNames = [
+        'T12',
+        'L2',
+        'L3',
+        'L3_TRUE',
+        'LL4',
+        'RL4',
+        'L4',
+        'LASI',
+        'RASI',
+        'RPSI',
+        'LPSI',
+        'SACR'
+    ]
+    points = {m: markers2[m] for m in pointNames}
+    pointsInPel = kine.changeMarkersReferenceFrame(points, RTPeli)
+    idxs = {}
+    for i in range(len(pointNames)):
+        pointName = pointNames[i]
+        idxName = 'idx_' + pointName
+        idxs[idxName] = i
+
+    colors = [
+        'b',
+        'b',
+        'b',
+        'g',
+        'b',
+        'b',
+        'b',
+        'r',
+        'r',
+        'r',
+        'r',
+        'r'
+    ]
+    
+    # Merge points in one array
+    pointsData = np.stack([pointsInPel[m] for m in pointNames], axis=2)  # Nf x 3 x Np
+    
+    # Init results
+    res = {}
+    res['spineAngles'] = {}
+    
+    # Create results directory if not existing
+    if not os.path.exists(resultsDir):
+        os.mkdir(resultsDir)
+    
+    Nf = pointsData.shape[0]
+    iRange = range(Nf)
+    if frames is not None:
+        iRange = frames
+        
+    if savePlots:
+        # Create results directory if not existing
+        figuresDir = os.path.join(resultsDir, 'figures')
+        if not os.path.exists(figuresDir):
+            os.mkdir(figuresDir)
+            
+    angleT12ToSACRToAntAxisSag = np.zeros((Nf,))
+    angleT12ToL4ToLatAxisFro = np.zeros((Nf,))
+        
+    # Process data
+    for i in iRange:
+        
+        print('processing time frame %d ...' % i)        
+        
+        # caculate data
+        pointsDataSag = pointsData[i,1::-1,:].T     # Np x 2
+        pointsDataFro = pointsData[i,1:,:].T        # Np x 2
+        
+        tmp = pointsDataSag[idxs['idx_T12'],:] - pointsDataSag[idxs['idx_SACR'],:]
+        angleT12ToSACRToAntAxisSag[i] = np.rad2deg(np.arctan2(tmp[1], tmp[0]))
+        
+        tmp = pointsDataSag[idxs['idx_T12'],:] - pointsDataSag[idxs['idx_L4'],:]
+        angleT12ToL4ToLatAxisFro[i] = np.rad2deg(np.arctan2(tmp[1], tmp[0]))
+        
+        if savePlots:
+        
+            plt.clf()
+                
+            plt.subplot(1, 2, 1)
+            for p in range(len(pointNames)):
+                plt.plot(pointsDataSag[p,1], pointsDataSag[p,0], 'o', c=colors[p])
+                plt.text(pointsDataSag[p,1]+5, pointsDataSag[p,0], pointNames[p])
+            plt.axis('equal')
+            plt.title('Sagittal')
+            plt.xlabel('X pelvis (anterior +)')
+            plt.ylabel('Y pelvis (up +)')
+            
+            plt.subplot(1, 2, 2)
+            for p in range(len(pointNames)):
+                plt.plot(pointsDataFro[p,1], pointsDataFro[p,0], 'o', c=colors[p])
+                plt.text(pointsDataFro[p,1], pointsDataFro[p,0], pointNames[p])
+            plt.axis('equal')
+            plt.title('Frontal')
+            plt.xlabel('Z pelvis (right +)')
+            
+            plt.savefig(os.path.join(figuresDir, 'tf_%04d.png' % i), format='png', orientation='landscape')
+        
+    if savePlots:
+        plt.close('all')
+        
+    res['newMarkers'] = markersNew
+    res['spineAngles']['T12ToSACRToAntAxisSag'] = angleT12ToSACRToAntAxisSag
+    res['spineAngles']['T12ToL4ToLatAxisFro'] = angleT12ToL4ToLatAxisFro
+    
+    return res
\ No newline at end of file
